package net.pricefx.common.apibuilder.chart;

import java.math.BigDecimal;

import net.pricefx.common.api.chart.ChartType;
import net.pricefx.common.api.chart.LimitPosition;
import net.pricefx.common.api.chart.LimitType;
import net.pricefx.common.api.chart.SortType;
import net.pricefx.common.api.chart.WhiskerType;
import net.pricefx.common.apibuilder.AbstractAdditionalMeasure;
import net.pricefx.common.apibuilder.AbstractBuilder;
import net.pricefx.common.apibuilder.AbstractOptions;
import net.pricefx.common.apibuilder.AbstractSeries;
import net.pricefx.common.apibuilder.IdGenerator;
import net.pricefx.common.apibuilder.Invocations;
import net.pricefx.common.apibuilder.MethodInvocation;
import net.pricefx.common.apibuilder.ObjectIdentity;



public class BoxplotChart extends AbstractBuilder {

	public BoxplotChart(String instanceId, String classId, ObjectIdentity parentChartObjectIdentity, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
		super(instanceId, classId, idGenerator, invocations);
	}

	private BoxplotOptions options;
	private Dictionary<BoxplotChart> dictionary;
	
	public BoxplotSeries addSeries() {

		String id = addInvocation("addSeries");
		return new BoxplotSeries(this, id, "Series", idGenerator, invocations);
	}
	
	public BoxplotOptions getOptions() {
		String id = addInvocation("getOptions");
		if (options == null) 
			options = new BoxplotOptions(this, id, "Options", idGenerator, invocations);
		return options;
	}
	
	public AuxLine<BoxplotChart> addAuxLine() {
		String id = addInvocation("addAuxLine");
		return new AuxLine<BoxplotChart>(this, id, "AuxLine");
	}
	
	public Dictionary<BoxplotChart> getDictionary() {
		String id = addInvocation("getDictionary");
		if (dictionary == null) 
			dictionary = new Dictionary<>(this, id, "Dictionary");
		return dictionary;
	}
	
	@Override
	public ChartBuilderResult build() {
		addInvocation(METHOD_BUILD);

		ChartBuilderResult r = new ChartBuilderResult();
		r.invocations = invocations.get();
		r.additionalInfo = ChartType.BOXPLOT;
		return r;
	}
	
	public static class BoxplotSeries extends AbstractSeries<BoxplotChart,BoxplotSeries> {

		public BoxplotSeries(BoxplotChart backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink, idGenerator, invocations);
		}

		
		public LimitToMeasure<BoxplotSeries> setLimitTo(LimitPosition limitPosition, BigDecimal amount, LimitType limitType) {
			String id = addInvocation("setLimitTo", limitPosition, amount, limitType);
			return new LimitToMeasure<BoxplotSeries>(this, id, "LimitToMeasure");
		}
		
		public BoxplotSeries setAxisX(String axisX) {
			addInvocation("setAxisX", axisX);
			return this;
		}
		
		public NonAggrFieldAxis<BoxplotSeries> setAxisY(String axisY) {
			String id = addInvocation("setAxisY", axisY);
			return new NonAggrFieldAxis<BoxplotSeries>(this, id, "AxisY");
		}

		public BoxplotSeries withSortByAxisX(SortType sortType) {
			addInvocation("withSortByAxisX", sortType);
			return this;
		}		
		
		public BoxplotSeries setComparisonDim(String comparisonDimension) {
			addInvocation("setComparisonDim", comparisonDimension);
			return this;
		}

		public BoxplotSeries withWhisker(WhiskerType whiskerType) {

			addInvocation("withWhisker", whiskerType);
			return this;
		}
		
		public BoxplotAdditionalMeasure<BoxplotSeries> addAdditionalMeasure() {
			String id = addInvocation("addAdditionalMeasure");
			return new BoxplotAdditionalMeasure<BoxplotSeries>(this, id, "AdditionalMeasure", idGenerator, invocations);
		}
		
		public Formula<BoxplotSeries> setFormula() {
			String id = addInvocation("setFormula");
			return new Formula<BoxplotSeries>(this, id, "Formula");
		}
		
		public BoxplotSeries setGeneratedQueryDto(String generatedQueryDto) {
			addInvocation("setGeneratedQueryDto", generatedQueryDto);
			return this;
		}


	}
	
	public static class BoxplotOptions extends AbstractOptions<BoxplotChart,BoxplotOptions> {

		public BoxplotOptions(BoxplotChart backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink, idGenerator, invocations);			
		}
				
	}
		
	public static class BoxplotAdditionalMeasure<ATTACHEDTO extends ObjectIdentity> extends AbstractAdditionalMeasure<ATTACHEDTO,BoxplotAdditionalMeasure<ATTACHEDTO>> {

		public BoxplotAdditionalMeasure(ATTACHEDTO attachedTo, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, attachedTo, idGenerator, invocations);
		}
		
		public BoxplotAdditionalMeasure<ATTACHEDTO> withField() {

			addInvocation("withField");
			return this;
		}
		
		public Expression<BoxplotAdditionalMeasure<ATTACHEDTO>> withExpression() { 
			
			String id = addInvocation("withExpression");
			return new Expression<BoxplotAdditionalMeasure<ATTACHEDTO>>(this, id, "Expression");
		}		
		

	}
	
}